/*
Copyright (c) 2011, Julian Lyndon-Smith (julian+maia@dotr.com)
http://www.dotr.com
All rights reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

using Progress.Lang.*.
using dotr.Maia.Model.*.

/** Maia Basic builder 
 * Core methods to build source file from the template provided
 * @author Julian Lyndon-Smith (julian+maia@dotr.com) 
 */

class dotr.Maia.Builder.Basic:

/** temptable to hold the meta-data for an object. 
 * Meta-data from each object (from the Maia Config directory) can 
 * hold information from a database (created using dotr/Maia/BuildTableMVC.w) 
 * and then enhanced with custom properties 
 */
     
  def protected temp-table TTProperty no-undo
    field PropertyName as char
    field fieldname as char
    field size as int
    field DataType as char
    field property as Progress.Lang.Object
    field DisplayInLookup as int
    field DisplayInForm as int
    field DisplayInGrid as int
    index Main is primary
      DataType
      PropertyName.

  /** holds all unique indicies from a table. 
   * Used to build get statements in the model and crud classes
   */
   
  def protected temp-table TTIndex no-undo
    field IndexName as char
    field Index1 as Progress.Lang.Object
    
    index Main is primary
      IndexName.
   
  def protected property ObjectConfig1 as ObjectConfig no-undo get . private set .
  
  def protected var DatatypeSize as  int  extent 10 no-undo.
  
  def public property BuilderAvailable as logical init yes no-undo get . protected set .
  
  def public property BuildDirectory as char no-undo get . protected set .
  def public property BuildPackage   as char no-undo get . protected set .
  
  /** p_ObjectConfig is the instance of the Maia Config/foo.xml file */
  constructor Basic (p_ObjectConfig as ObjectConfig):
    assign ObjectConfig1 = p_ObjectConfig.
    
    /** init activerecord template stuff */
    Init().
    
    /** init the properties of the supplied object */
    BuildProperties().
    
  end constructor.

  method private void Init():
    def var lv_Path    as char no-undo.
    def var lv_Project as char no-undo.
    
    def var ObjectBuilder as dotr.Maia.Builder.ObjectBuilder no-undo.
    
    assign lv_Project    = dotr.Maia.CurrentProject:Name
           lv_Path       = dotr.Maia.CurrentProject:Path
           ObjectBuilder = dotr.Maia.Builder.ObjectBuilder:Instance.
           
    ObjectBuilder:SetDefaultOption("ModelClass",ObjectConfig1:ClassName).
    ObjectBuilder:SetDefaultOption("ModelPackage",substitute("&1.Model",lv_Project)).  
    ObjectBuilder:SetDefaultOption("ModelDirectory",substitute("&1/Model",lv_Path)). 
    
    ObjectBuilder:SetDefaultOption("HelperPackage",substitute("&1.Helper",lv_Project)).  
    ObjectBuilder:SetDefaultOption("HelperDirectory",substitute("&1/Helper",lv_Path)).
    
    ObjectBuilder:SetDefaultOption("LookupClass",substitute("&1Lookup",ObjectConfig1:ClassName)).
    
    ObjectBuilder:SetDefaultOption("CollectionClass",substitute("&1Collection",ObjectConfig1:ClassName)).
               
    os-create-dir value(lv_Path).
    os-create-dir value(substitute("&1/_Maia-Config",lv_Path)).
    os-create-dir value(substitute("&1/_Maia-Template",lv_Path)).
    os-create-dir value(substitute("&1/Helper",lv_Path)).
    os-create-dir value(substitute("&1/Helper/Base",lv_Path)).
    os-create-dir value(substitute("&1/Model",lv_Path)).
    os-create-dir value(substitute("&1/Model/Base",lv_Path)).
    os-create-dir value(substitute("&1/UI",lv_Path)).
    os-create-dir value(substitute("&1/UI/Images",lv_Path)).
    os-create-dir value(substitute("&1/UI/Library",lv_Path)).
    os-create-dir value(substitute("&1/UI/winform",lv_Path)).
    os-create-dir value(substitute("&1/UI/winform/Ribbon",lv_Path)).
    os-create-dir value(substitute("&1/UI/winform/Ribbon/Base",lv_Path)).
    os-create-dir value(substitute("&1/UI/winform/Lookup",lv_Path)).
    os-create-dir value(substitute("&1/UI/winform/Lookup/Base",lv_Path)).
    os-create-dir value(substitute("&1/UI/winform/Update",lv_Path)).
    os-create-dir value(substitute("&1/UI/winform/Update/Base",lv_Path)).
  end method.
  


  /** Dummy placeholder for start of build event
   * @param p_Data : Contents of template
   * @return longchar: altered data
   */
   
  method public longchar StartBuild(p_Data as longchar):
    return p_Data.
  end method.

  /** Dummy placeholder for end of build event
   * @param p_Data : Contents of modified template
   * @return longchar: altered data
   */

  method public longchar EndBuild(p_Data as longchar):
    return p_Data.
  end method.
  
  /** Example of how to write a build method. Not a very useful method ;)
   * @param p_Directive : holds the directive name and all parameters
   */
  method public void Using(p_Directive as Directive):
    def var lv_i as int no-undo.
    def var lv_j as int no-undo.

    /** loop through all parameters of directive */    
    do lv_i = 1 to extent(p_Directive:ParameterData):
      do lv_j = 1 to num-entries(p_Directive:ParameterData[lv_i]):
        assign p_Directive:Data = p_Directive:Data + substitute("using &1~n",entry(lv_j,p_Directive:ParameterData[lv_i])).
      end.
    end.
    
  end method.
  
  /** get the file contents of selected file 
   *  @param p_Directive Directive
   *  @param p_File default file to use
   */
  method private longchar GetFileContents(p_Directive as Directive,p_File as char):
    def var lv_Header as longchar no-undo.
    
    def var lv_File as char no-undo.
    
    assign lv_File             = if GetParameter(p_Directive,"Source") eq "" then p_File else GetParameter(p_Directive,"Source")
           file-info:file-name = search(substitute("&1/&2",dotr.Maia.CurrentProject:TemplateDirectory,lv_File)).
           
    if file-info:full-pathname eq ? then assign file-info:file-name = search(substitute("dotr/Maia/Template/&1",lv_File)).                                                 
    
    if file-info:full-pathname eq ? then return "".

    copy-lob from file file-info:full-pathname to lv_Header.
    
    return lv_header.

  end method.

  /** Example of how to write a build method. Not a very useful method ;)
   * Gives example of how to use parameter if supplied, or default to standard option
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void MethodHeader(p_Directive as Directive):
    def var lv_i as int no-undo.

    def var lv_Header as longchar no-undo.
    
    assign lv_Header = GetFileContents(p_Directive,"builder/MethodHeader.i").
    
    /** loop through all parameters of directive
     * all options are used to substitute <<option>> tags in the supplied parameters */    
    do lv_i = 1 to extent(p_Directive:ParameterData):
      case p_Directive:ParameterName[lv_i]:
        otherwise assign lv_Header = replace(lv_Header, substitute("<<&1>>",p_Directive:ParameterName[lv_i]),p_Directive:ParameterData[lv_i]). 
      end case.
    end.
    
    assign p_Directive:Data = lv_Header. 
    
  end method.

  /** create statement to duplicate a model's properties
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void Duplicate(p_Directive as Directive):
    def var lv_Data   as char no-undo.
    def var lv_Assign as char no-undo.
    
    def var lv_maxLen as int no-undo.
    
    def var Property1 as Property no-undo.
    
    assign lv_maxLen = length("PrivateData").
    
    /** get max size of property name. Used to align code properly */
    for each TTProperty no-lock :
      Property1 = cast(TTProperty.Property,Property).

      /** a manual property has no db relation , so skip it*/
      if Property1:ManualProperty then next.
      
      if Property1:BuildProperty then assign lv_MaxLen = MAX(lv_MaxLen,TTProperty.SIZE).
    end.
    
    /** now let's loop through, building the assign statement */
    for each TTProperty no-lock:
      Property1 = cast(TTProperty.Property,Property).
      
      if not Property1:BuildProperty then next.
      
      assign lv_Assign = substitute("this-object:&1@ = p_<<ModelClass>>:&1~n",Property1:PropertyName)
             lv_Data   = lv_Data + replace(lv_Assign,"@",fill(" ",lv_maxLen - TTProperty.Size)). 
    end.

    assign lv_Assign = "this-object:rowid@ = p_<<ModelClass>>:rowid~n"
           lv_Data   = lv_Data + replace(lv_Assign,"@",fill(" ",lv_maxLen - length("rowid"))) 
           lv_Assign = "this-object:PrivateData@ = p_<<ModelClass>>:PrivateData~n"
           lv_Data   = lv_Data + replace(lv_Assign,"@",fill(" ",lv_maxLen - length("PrivateData"))). 

    assign p_Directive:Data = trim(lv_Data,"~n") + ".".                                                                                                                                                  
  end method.
  

  /** Generate JavaDoc method header comment
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void JavaDocHeader(p_Directive as Directive):
    def var lv_i as int no-undo.
    
    def var lv_Data as char no-undo.
    
    def var lv_Header as longchar no-undo.
    
    assign file-info:file-name = if GetParameter(p_Directive,"Source") eq "" then "dotr/Maia/Template/builder/JavaDocHeader.i"
                                                                             else GetParameter(p_Directive,"Source").                                  
      
    copy-lob from file file-info:full-pathname to lv_Header.
    
    /** loop through all parameters of directive
     * all options are used to substitute <<option>> tags in the supplied parameters 
     * the \n tag is used to insert  new line in the comment      
     * @param p_Directive : holds the directive name and all parameters
     */
    do lv_i = 1 to extent(p_Directive:ParameterData):
      assign lv_Data = p_Directive:ParameterData[lv_i]
             lv_Data = replace(lv_Data,"\n","~n * ").
             
      case p_Directive:ParameterName[lv_i]:
        otherwise assign lv_Header = replace(lv_Header, substitute("<<&1>>",p_Directive:ParameterName[lv_i]),lv_Data). 
      end case.
    end.
    
    assign lv_header = replace(lv_header," * <<LongDescription>>~n","") /** no longdescription supplied */ 
           lv_header = replace(lv_header," * <<Parameters>>~n","") /** no parameters supplied */ 
           lv_header = replace(lv_header," * <<ReturnValue>>~n","") /** no ReturnValue supplied */ 
           p_Directive:Data = lv_Header. 
    
  end method.

  /** insert custom code
   * calls the getCustomCode method in the objectConfig class to insert custom code into the template
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void CustomCode(p_Directive as Directive):
    def var lv_i as int no-undo.

    def var CustomCode1 as CustomCode no-undo.
    
    /** loop through all parameters of directive     */
    do lv_i = 1 to extent(p_Directive:ParameterData):
      CustomCode1 = ObjectConfig1:GetCustomCode(p_Directive:ParameterData[lv_i]).
      
      if CustomCode1:CodeName eq "" then next.
        
      assign p_Directive:Data = p_Directive:Data + "~n" + CustomCode1:Data. 
    end.
 
    assign p_Directive:Data = trim(p_Directive:Data,"~n"). 
 
  end method.

  /** build the "def property" statement for each of the properties
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void DefineProperties(p_Directive as Directive):
    def var lv_i      as int no-undo.
    def var lv_Index  as int no-undo.
    def var lv_MaxLen as  int  no-undo extent 10.
      
    def var lv_DataType    as char no-undo.
    def var lv_PropertyDef as char no-undo init "&6def &5 property &1@ as &2 no-undo &3 get . set . &4".
    def var lv_Property    as char no-undo.
    def var lv_Comments    as char no-undo.
    
    def var Property1 as Property no-undo.

    /** loop through all properties
     * break by datatype, work out the length of the longest property name.
     * this is done so that the code can look pretty */
     
    for each TTProperty no-lock break by TTProperty.DataType by TTProperty.PropertyName:
      if first-of(TTProperty.DataType) then assign lv_Index = lv_index + 1.
      assign lv_MaxLen[lv_Index] = MAX(lv_MaxLen[lv_Index],TTProperty.SIZE).
    end.
    
    assign lv_Index = 0.

    /** loop through all properties again
     * this time, generating the actual code. Spacers are placed to align the code */
          
    for each TTProperty no-lock break by TTProperty.DataType by TTProperty.PropertyName:
      if first-of(TTProperty.DataType) then assign lv_Index = lv_index + 1.
      
      Property1 = cast(TTProperty.Property,Property).
      
      /** don't build invalid properties */
      if not valid-object(Property1) or Property1:PropertyName eq "" then next.
      if not Property1:BuildProperty then next.
      
      assign lv_Comments = "".
       
      if Property1:Comment ne "" or Property1:Brief ne "" then
      do:
        assign lv_Comments = substitute("  /** &1",Property1:Brief).
        
        do lv_i = 1 to num-entries(Property1:Comment,"~n"):
          assign lv_Comments = substitute("&1~n   * &2",entry(lv_i,lv_Comments,"~n")).
        end.
        
        assign lv_Comments = lv_Comments + " */".
      end.
      
      assign lv_Property      = substitute(lv_PropertyDef,TTProperty.PropertyName, 
                                                          if num-entries(Property1:datatype,".") gt 1 then Property1:datatype else lc(Property1:datatype), 
                                                          if Property1:Extent gt 0 or Property1:extent eq ? then substitute("extent &1",if Property1:extent eq ? then "" else string(Property1:extent)) else "",
                                                          if Property1:Comment ne "" then substitute("/* &1 */",Property1:Comment) else "",
                                                          lc(Property1:AccessMode))
                                                          
             lv_Property      = replace(lv_Property,"@",fill(" ",lv_maxLen[lv_Index] - TTProperty.Size)) 
             p_Directive:Data = p_Directive:Data + "~n" + lv_Property.

      /** insert blank line for each break of data type */
      if last-of(TTProperty.DataType) then p_Directive:Data = p_Directive:Data + "~n".
    end.
    
    assign p_Directive:Data = left-trim(p_Directive:Data,"~n").
  end method.
  
  /** build the methods for each of the get statements in the model
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void DefineGetMethods(p_Directive as Directive):
    def var lv_i      as int no-undo.

    def var lv_Index as char no-undo.

    def var lv_Header as longchar no-undo.
    
    /** check for custom builder file */
    assign file-info:file-name = if GetParameter(p_Directive,"Source") eq "" then "dotr/Maia/Template/builder/getModelMethod.i"
                                                                             else GetParameter(p_Directive,"Source").                                  
    
    copy-lob from file file-info:full-pathname to lv_Header.
    
    def var Index1 as Index no-undo.

    /** get the find statements */
    BuildIndex().

    /** build get methods */
    for each TTIndex no-lock break by TTIndex.IndexName:

      Index1 = CAST(TTIndex.Index1,Index).
      
      if not valid-object(Index1) or Index1:IndexName eq "" then next.
      
          assign lv_Index = replace(lv_Header,"<<GetName>>",Index1:IndexName)
                 lv_index = replace(lv_index,"<<DocParameterList>>",replace(Index1:parametersIn," as ", " - "))
                 lv_index = replace(lv_index,"<<FindString>>",Index1:FindString)
                 lv_index = replace(lv_index,"<<ParameterList>>",Index1:parametersIn)
                 lv_index = replace(lv_index,"<<ParameterCall>>",Index1:parametersCall)
    
                 p_Directive:Data = p_Directive:Data + "~n" + lv_Index.
    end.
    
  end method.

  /** fluent method definitions
   * experimental feature to make code "easier" to read
   * each property can have a "fluent" method name, which takes a parameter to set the property
   * but then returns THIS-OBJECT so that you can chain method calls together
   *
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void DefineFluentMethods(p_Directive as Directive):
    def var lv_i      as int no-undo.

    def var lv_Data as char no-undo.

    def var lv_Header as longchar no-undo.
    
    def var Property1 as Property no-undo.
    
    /** check for custom builder file */
    assign file-info:file-name = if GetParameter(p_Directive,"Source") eq "" then "dotr/Maia/Template/builder/fluent.i"
                                                                             else GetParameter(p_Directive,"Source").                                  
    
    copy-lob from file file-info:full-pathname to lv_Header.
    
    /** loop through all properties, building fluent method calls for that property */ 
    for each TTProperty no-lock break by TTProperty.DataType by TTProperty.PropertyName:
      
      if TTProperty.PropertyName eq ObjectConfig1:KeyProperty then next.
       
      Property1 = cast(TTProperty.Property,Property).

      assign lv_Data = replace(lv_Header,"<<FluentName>>",Property1:Fluent)
             lv_Data = replace(lv_Data,"<<PropertyName>>",TTProperty.PropertyName)
             lv_Data = replace(lv_Data,"<<DataType>>",TTProperty.DataType) 

             p_Directive:Data = p_Directive:Data + "~n" + lv_Data.
    end.
    
  end method.

  /** Insert copyright message in each file
   * @param p_Directive : holds the directive name and all parameters
   */
  method public void Copyright(p_Directive as Directive):
    assign p_Directive:Data = GetFileContents(p_Directive,"Builder/Copyright.i").
  end method.

  /** Insert version control ID tag
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void ID(p_Directive as Directive):
    assign p_Directive:Data = "/* $Id$ */".
  end method.
  
  /** create "instance" statements
   * used to create pseudo-singletons. (Singletons without the static defintions 
   * @param p_Directive : holds the directive name and all parameters
   */
  method public void DefineInstance(p_Directive as Directive):
    def var lv_Data as longchar no-undo.
    
    assign file-info:file-name = if GetParameter(p_Directive,"Source") eq "" then "dotr/Maia/Template/builder/instance.i" 
                                                                             else GetParameter(p_Directive,"Source").
    
    copy-lob from file file-info:full-pathname to lv_Data.
    
    assign p_Directive:Data = lv_Data. 
  end method.

  /** class event definitions
   * @param p_Directive : holds the directive name and all parameters
   */
  method public void Publish(p_Directive as Directive):
    def var lv_Data as longchar no-undo.
    
    assign file-info:file-name = if GetParameter(p_Directive,"Source") eq "" then "dotr/Maia/Template/builder/publish.i" 
                                                                           else GetParameter(p_Directive,"Source").

    copy-lob from file file-info:full-pathname to lv_Data.
    
    assign p_Directive:Data = replace(lv_Data,"<<EventName>>",GetParameter(p_Directive,"Name")).
  end method.
  
  /** Example of how to write a build method. Not a very useful method ;)
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void DefineDefaultconstructor(p_Directive as Directive):
    assign p_Directive:Data = "constructor <<ClassName>>():~nend constructor.".    
  end method.

  /** create a buffer, using the database name if supplied
   * @param p_Directive : holds the directive name and all parameters
   */
  method public void DefineBuffer(p_Directive as Directive):
    def var lv_db as char no-undo.
    
    assign lv_db = GetParameter(p_Directive,"DB") .
    
    if lv_db ne "" then assign lv_Db = lv_db + ".".
    
    assign p_Directive:Data = substitute("def buffer &1 for &2&1.",ObjectConfig1:tableName,lv_db).    
  end method.

  /** gets the value of a directive parameter
   * @param p_Directive : holds the directive name and all parameters
   * @return char : Value of parameter. Blank if not found
   */
   
  method protected char GetParameter(p_Directive as Directive,p_Label as char):
    def var lv_i as int no-undo.
    
    do lv_i = 1 to extent(p_Directive:ParameterData):
      if p_Directive:ParameterName[lv_i] eq p_Label then return p_Directive:ParameterData[lv_i].
    end.
    
    return "".
  end method.  

  /** Create temp-table record for each property */

  method public void BuildProperties():
    empty temp-table TTProperty.
    
    def var lv_i as int no-undo.
    
    def var fkObject   as dotr.Maia.Model.ObjectConfig no-undo. 
    def var fkProperty as dotr.Maia.Model.Property     no-undo. 
    
    do lv_i = 1 to ObjectConfig1:NumProperties:
      if not valid-object(ObjectConfig1:Property[lv_i]) then next.
      if ObjectConfig1:Property[lv_i]:PropertyName eq "" then next.
      
      create TTProperty.
      
      assign TTProperty.DataType        = ObjectConfig1:Property[lv_i]:DataType
             TTProperty.property        = ObjectConfig1:Property[lv_i]
             TTProperty.PropertyName    = ObjectConfig1:Property[lv_i]:PropertyName
             TTProperty.FieldName       = ObjectConfig1:Property[lv_i]:FieldName
             TTProperty.DisplayInForm   = ObjectConfig1:Property[lv_i]:DisplayInForm
             TTProperty.DisplayInLookup = ObjectConfig1:Property[lv_i]:DisplayInLookup
             TTProperty.DisplayInGrid   = ObjectConfig1:Property[lv_i]:DisplayInGrid
             TTProperty.SIZE            = length(TTProperty.PropertyName).
             
      /** create a property for the foreign key */
      if ObjectConfig1:Property[lv_i]:isForeignKey then
      do:
        create TTProperty.

        assign TTProperty.DataType        = ObjectConfig1:Property[lv_i]:ForeignKey:DataType
               TTProperty.property        = ObjectConfig1:Property[lv_i]:ForeignKey
               TTProperty.PropertyName    = ObjectConfig1:Property[lv_i]:fkModelName
               TTProperty.SIZE            = length(TTProperty.PropertyName).
      end.             
    end.
        
  end method.

  /** creates temp-table records for all unique indicies of an object */
  
  method public void BuildIndex():
    empty temp-table TTIndex.
    
    def var lv_i as int no-undo.
    
    do lv_i = 1 to ObjectConfig1:NumIndex:
      if not valid-object(ObjectConfig1:Index[lv_i]) then next.
      if ObjectConfig1:Index[lv_i]:IndexName eq "" then next.
      
      create TTIndex.
      
      assign TTIndex.Index1    = ObjectConfig1:Index[lv_i]
             TTIndex.IndexName = ObjectConfig1:Index[lv_i]:IndexName.
    end.
        
  end method.
      
end class.