/*
Copyright (c) 2011, Julian Lyndon-Smith (julian+maia@dotr.com)
http://www.dotr.com
All rights reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

using Progress.Lang.*.
using dotr.Maia.Model.*.

/** Maia ribbon builder 
 * extends the basic builder to add methods to build a ribbon class from the BaseRibbon.cls template provided
 * @author Julian Lyndon-Smith (julian+maia@dotr.com) 
 */

class dotr.Maia.Builder.BuildBaseRibbon inherits dotr.Maia.Builder.Basic:
  def var numFields as int no-undo.

  /** create directory structure if required
   */
        
  constructor BuildBaseRibbon (p_ObjectConfig as ObjectConfig):
    def var ObjectBuilder as dotr.Maia.Builder.ObjectBuilder no-undo.
    
    super(p_ObjectConfig).
    
    assign ObjectBuilder = dotr.Maia.Builder.ObjectBuilder:Instance.
    
    assign BuildPackage   = substitute("&1.UI.winform.Ribbon.base",dotr.Maia.CurrentProject:Name)
           BuildDirectory = substitute("&1/UI/winform/Ribbon/base",dotr.Maia.CurrentProject:Path).
    
    /** always display keyfield in the grid */
    find first ttproperty where ttproperty.PropertyName eq ObjectConfig1:KeyProperty .
    
    assign ttproperty.DisplayInGrid = 1.
    
    /** count the number of fields to show in the card and grid view */
    for each ttproperty no-lock where TTProperty.DisplayInGrid gt 0 numfields = 1 to numfields + 1:
    end.
    
  end constructor.

  /** inject generator code into the template
   * any code manually placed in the initializecomponents method is overwritten
   * this method takes the template, looks for the appropriate places
   * and injects generator code into the template, after it has been read from disk
   * this makes sure that no code is lost if the template is loaded into the visual designer
   * 
   * @param p_Data : code for the template
   * @return longchar : the modified template
   */

  method public override longchar StartBuild(p_Data as longchar):
    def var lv_resx    as longchar no-undo.
    def var lv_inherit as longchar no-undo.
    
    def var lv_RibbonBase as char no-undo.
    
    /** replace all PlaceHolder phrase with the name of the object
      * Placeholder is used so that we can actually load the template
      * into the visual designer and modify it if required */
      
    assign lv_RibbonBase = substitute("&1/UI/winform/Ribbon/Base/&2",dotr.Maia.CurrentProject:Name,ObjectConfig1:CurrentClassName)
    
           p_Data = replace(p_Data, "dotr.Maia.Model.PlaceHolder","<<ModelPackage>>.<<ModelClass>>")
           
           p_Data = replace(p_Data,"dotr.Maia.UI.winform.Update.PlaceHolder",substitute("&1.UI.winform.Update.&2Update",replace(dotr.Maia.CurrentProject:Name,"/","."),ObjectConfig1:ClassName))
           p_Data = replace(p_Data,"dotr/Maia/Template/BaseRibbon",lv_RibbonBase)
       
           p_Data = replace(p_Data,"dotr.Maia.Template.BaseRibbon",replace(lv_RibbonBase,"/","."))
           p_Data = replace(p_Data," BaseRibbon ",substitute(" &1 ",ObjectConfig1:CurrentClassName))
           
           p_Data = replace(p_Data, "PlaceHolder","<<ModelClass>>")
           p_Data = replace(p_Data, "dotr.Maia:","<<Package>>.<<Package>>:")
           
           p_Data = replace(p_Data,"THIS-OBJECT:Load:Subscribe(THIS-OBJECT:<<ModelClass>>_Load).",
                                   "~{&! UITextNew !}~n" +
                                   '@VisualDesigner.FormMember (NeedsInitialize="false", InitializeArray="true").~n' +
                                   "~{&! UIAddCardArray !}~n" +
                                   "~{&! GridProperties !}~n" +
                                   '@VisualDesigner.FormMember (NeedsInitialize="false", InitializeArray="true").~n' +
                                   "~{&! UIAddGridArray !}~n" +
                                   "~{&! CardProperties !}~n" +
                                   "~{&! DataSchema !}~n" +
                                   "THIS-OBJECT:Load:Subscribe(THIS-OBJECT:<<ModelClass>>_Load).").
         
    os-copy value(search("dotr/Maia/Template/BaseRibbon.resx")) value(substitute("&1.resx",lv_RibbonBase)).
                   
    return p_Data.
  end method.

  /** create the define variable statements required
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void UITextDefine(p_Directive as Directive):
    def var i as int.
    def var lv_Data as char no-undo.
    
    for each TTProperty no-lock where TTProperty.DisplayInGrid gt 0 by TTProperty.PropertyName i = 1 to i + 1:
      assign lv_Data = lv_Data + substitute("def protected var col&1  AS DevExpress.XtraGrid.Columns.GridColumn NO-UNDO.~n",replace(TTProperty.PropertyName,"-",""))
             lv_Data = lv_Data + substitute("def protected var col&11 AS DevExpress.XtraGrid.Columns.GridColumn NO-UNDO.~n",replace(TTProperty.PropertyName,"-","")).
    end.

    assign p_Directive:Data = trim(lv_Data,"~n").
  end method.
  
  /** create the new object statements required
   * @param p_Directive : holds the directive name and all parameters
   */
  method public void UITextNew(p_Directive as Directive):

    def var lv_Data as char no-undo.
    
    for each TTProperty no-lock where TTProperty.DisplayInGrid gt 0 by TTProperty.PropertyName:
      assign lv_Data = lv_Data + substitute("THIS-OBJECT:col&1  = NEW DevExpress.XtraGrid.Columns.GridColumn().~n",replace(TTProperty.PropertyName,"-",""))
             lv_Data = lv_Data + substitute("THIS-OBJECT:col&11 = NEW DevExpress.XtraGrid.Columns.GridColumn().~n",replace(TTProperty.PropertyName,"-","")).
    end.
    
    assign p_Directive:Data = trim(lv_Data,"~n").
  end method.

  /** create the statements required to add fields to the card view 
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void UIAddCardArray(p_Directive as Directive):

    def var lv_Data as char no-undo.
    
    def var lv_i as int no-undo.
    
    assign lv_Data = substitute('DEFINE VARIABLE arrayvar9 AS DevExpress.XtraGrid.Columns.GridColumn EXTENT &1 NO-UNDO.~n',
                                NumFields).
    
    for each TTProperty no-lock where TTProperty.DisplayInGrid gt 0 by TTProperty.PropertyName lv_i = 1 to lv_i + 1 :
      assign lv_Data = lv_Data + substitute("arrayvar9[&2] = THIS-OBJECT:col&1.~n",replace(TTProperty.PropertyName,"-",""),lv_i).
    end.
    
    assign lv_Data = lv_Data + substitute('THIS-OBJECT:cardView1:Columns:AddRange(arrayvar9).~n',ObjectConfig1:ClassName).
    
    assign p_Directive:Data = trim(lv_Data,"~n").
  end method.

  /** create the statements required to add fields to the grid view 
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void UIAddGridArray(p_Directive as Directive):

    def var lv_Data as char no-undo.
    
    def var lv_i as int no-undo.
    
    assign lv_Data = substitute('DEFINE VARIABLE arrayvar10 AS DevExpress.XtraGrid.Columns.GridColumn EXTENT &1 NO-UNDO.~n',
                                NumFields).
    
    for each TTProperty no-lock where TTProperty.DisplayInGrid gt 0 by TTProperty.PropertyName lv_i = 1 to lv_i + 1 :
      assign lv_Data = lv_Data + substitute("arrayvar10[&2] = THIS-OBJECT:col&11.~n",replace(TTProperty.PropertyName,"-",""),lv_i).
    end.
    
    assign lv_Data = lv_Data + "THIS-OBJECT:gridView1:Columns:AddRange(arrayvar10).~n".
    
    assign p_Directive:Data = trim(lv_Data,"~n").
  end method.

  /** create the statements required to create the data schema 
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void DataSchema(p_Directive as Directive):
    def var lv_Data as char no-undo.
    
    def var lv_i as int no-undo.

    if NumFields eq 0 then /** nothing to do */
    do:
     assign p_Directive:Data = "".
     return.
    end.
    
    assign lv_Data = substitute('@VisualDesigner.FormMember (NeedsInitialize="false", InitializeArray="true").~n' +
                                'def var arrayvar7 AS "Progress.Data.TableDesc[]" NO-UNDO.~n' + 
                                'arrayvar7 = NEW "Progress.Data.TableDesc[]"(0).~n' +
                                'DEFINE VARIABLE tableDesc1 AS Progress.Data.TableDesc NO-UNDO.~n' +
                                'tableDesc1 = NEW Progress.Data.TableDesc("&2").~n' +
                                'tableDesc1:ChildTables = arrayvar7.~n' +
                                '@VisualDesigner.FormMember (NeedsInitialize="false", InitializeArray="true").~n' +
                                'DEFINE VARIABLE arrayvar8 AS Progress.Data.ColumnPropDesc EXTENT &1 NO-UNDO.~n',
                                NumFields,
                                ObjectConfig1:ClassName).
                     
    for each TTProperty no-lock where TTProperty.DisplayInGrid gt 0 by TTProperty.DisplayInGrid lv_i = 1 to lv_i + 1:
      assign lv_Data = lv_Data + substitute('arrayvar8[&1] = new Progress.Data.ColumnPropDesc("&3", "&2", Progress.Data.DataType:&4).~n',
                                            lv_i,
                                            TTProperty.PropertyName,
                                            TTProperty.FieldName,
                                            TTProperty.DataType).  
    end.
    
    assign lv_Data = lv_Data + 'tableDesc1:Columns = arrayvar8.~n' +
                               'THIS-OBJECT:bindingSource1:TableSchema = tableDesc1.~n'.
    
        
    assign p_Directive:Data = trim(lv_Data,"~n").
  end method.

  /** create the statements required to show the fields in the grid 
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void GridProperties(p_Directive as Directive):
    def var lv_Data as char no-undo.
    
    def var lv_i as int no-undo.

    if NumFields eq 0 then /** nothing to do */ 
    do:
     assign p_Directive:Data = "".
     return.
    end.
    
    for each TTProperty no-lock where TTProperty.DisplayInGrid gt 0 by TTProperty.DisplayInGrid lv_i = 0 to lv_i + 1:
      assign lv_Data = lv_Data + substitute('/*  */~n' +
                                            '/* col&11 */~n' +
                                            '/*  */~n' +
                                            'THIS-OBJECT:col&51:Caption = "&1".~n' +
                                            'THIS-OBJECT:col&51:FieldName = "&2".~n' +
                                            'THIS-OBJECT:col&51:Name = "col&51".~n' +
                                            'THIS-OBJECT:col&51:Visible = &4.~n' +
                                            'THIS-OBJECT:col&51:VisibleIndex = &3.~n',
                                            
                                            TTProperty.PropertyName,
                                            TTProperty.FieldName,
                                            if TTProperty.PropertyName eq ObjectConfig1:KeyProperty 
                                               then -1 
                                               else TTProperty.DisplayInGrid - 1,
                                            if TTProperty.PropertyName eq ObjectConfig1:KeyProperty
                                               then "FALSE" else "TRUE",
                                            replace(TTProperty.PropertyName,"-","")).                                            
    end.
      
    assign p_Directive:Data = trim(lv_Data,"~n").
  end method.

  /** create the statements required to show the fields in the card
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void CardProperties(p_Directive as Directive):
    def var lv_Data as char no-undo.
    
    def var lv_i as int no-undo.

    if NumFields eq 0 then /** nothing to do */ 
    do:
     assign p_Directive:Data = "".
     return.
    end.
    
    for each TTProperty no-lock where TTProperty.DisplayInGrid gt 0 by TTProperty.DisplayInGrid lv_i = 0 to lv_i + 1:
      
      assign lv_Data = lv_Data + substitute('/*  */~n' +
                                            '/* col&1 */~n' +
                                            '/*  */~n' +
                                            'THIS-OBJECT:col&5:Caption = "&1".~n' +
                                            'THIS-OBJECT:col&5:FieldName = "&2".~n' +
                                            'THIS-OBJECT:col&5:Name = "col&5".~n' +
                                            'THIS-OBJECT:col&5:Visible = &4.~n' +
                                            'THIS-OBJECT:col&5:VisibleIndex = &3.~n',
                                            
                                            TTProperty.PropertyName,
                                            TTProperty.FieldName,
                                            if TTProperty.PropertyName eq ObjectConfig1:KeyProperty 
                                               then -1 
                                               else TTProperty.DisplayInGrid - 1,
                                            if TTProperty.PropertyName eq ObjectConfig1:KeyProperty
                                               then "FALSE" else "TRUE",
                                            replace(TTProperty.PropertyName,"-","")).                                            
    end.
      
    assign p_Directive:Data = trim(lv_Data,"~n").
  end method.

  /** create the statement to get the keyfield when the grid or cad is double-clicked 
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void RibbonGet(p_Directive as Directive):
    assign p_Directive:Data = substitute('when "gridView1" then return (new <<ModelPackage>>.<<ModelClass>>()):find(&1(gridview1:GetRowCellValue(gridView1:FocusedRowHandle,"<<KeyFieldName>>"))).~n' +
                                         'when "cardView1" then return (new <<ModelPackage>>.<<ModelClass>>()):find(&1(cardView1:GetRowCellValue(cardView1:FocusedRowHandle,"<<KeyFieldName>>"))).',
                                         if ObjectConfig1:KeyFieldDatatype begins "int" then "unbox" else "string"). 
  end method.
  

end class.