/*
Copyright (c) 2011, Julian Lyndon-Smith (julian+maia@dotr.com)
http://www.dotr.com
All rights reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

using Progress.Lang.*.
using dotr.Maia.Model.*.

/** builder for the update screens for an object
 * 2 classes are created, one for the base which contains all the update and assign statements
 * and one that inherits the base so that visual customizations can be implemented  
 * @author Julian Lyndon-Smith (julian+maia@dotr.com) 
 */

class dotr.Maia.Builder.BuildBaseUpdate inherits dotr.Maia.Builder.Basic:
  def protected var NumFields as int no-undo.
  
  def var UIBuilder as dotr.Maia.Builder.DevExpress no-undo.
  
  def var BeginInit  as longchar no-undo.
  def var EndInit    as longchar no-undo.
  def var textDefine as longchar no-undo.
  def var textNew    as longchar no-undo.
  def var AddControl as longchar no-undo.
  
  constructor  BuildBaseUpdate(p_ObjectConfig as ObjectConfig):
    def var ObjectBuilder as dotr.Maia.Builder.ObjectBuilder no-undo.
    
    super(p_ObjectConfig).
    
    assign ObjectBuilder = dotr.Maia.Builder.ObjectBuilder:Instance.
    
    assign BuildPackage =  substitute("&1.UI.winform.Update.base",dotr.Maia.CurrentProject:Name)
           BuildDirectory = substitute("&1/UI/winform/Update/base",dotr.Maia.CurrentProject:Path).
    Init().
  end constructor.

  /** get all data required to fill template in with
   */
  
  method private void init():  
    def var Property1 as dotr.Maia.Model.Property no-undo.
  
    def var lv_Data as char no-undo.

    UIBuilder = new dotr.Maia.Builder.DevExpress().
    
    for each TTProperty no-lock where TTProperty.DisplayInForm gt 0 NumFields = 1 to NumFields + 1:
      Property1 = cast(TTProperty.property,dotr.Maia.Model.Property).

      assign BeginInit  = BeginInit  + UIBuilder:ComponentBuilder(Property1:UIType):BeginInit(replace(Property1:PropertyName,"-",""))  
             EndInit    = EndInit    + UIBuilder:ComponentBuilder(Property1:UIType):EndInit(replace(Property1:PropertyName,"-","")) 
             textDefine = textDefine + UIBuilder:ComponentBuilder(Property1:UIType):Definition(replace(Property1:PropertyName,"-",""))
             AddControl = AddControl + UIBuilder:ComponentBuilder(Property1:UIType):AddControl(replace(Property1:PropertyName,"-",""))
             textNew    = textNew    + UIBuilder:ComponentBuilder(Property1:UIType):New(replace(Property1:PropertyName,"-","")).
    end.
    
    assign BuilderAvailable = numFields gt 0 . 
    
  end method.

  /** inject generator code into the template
   * any code manually placed in the initializecomponents method is overwritten
   * this method takes the template, looks for the appropriate places
   * and injects generator code into the template, after it has been read from disk
   * this makes sure that no code is lost if the template is loaded into the visual designer
   * 
   * @param p_Data : code for the template
   * @return longchar : the modified template
   */

  method public override longchar StartBuild(p_Data as longchar):
    def var lv_resx    as longchar no-undo.
    
    def var lv_UpdateBase as char no-undo.
    
    /** replace all PlaceHolder phrase with the name of the object
      * Placeholder is used so that we can actually load the template
      * into the visual designer and modify it if required */
     
    assign lv_UpdateBase = substitute("&1/UI/winform/Update/Base/&2",dotr.Maia.CurrentProject:Name,ObjectConfig1:CurrentClassName)
           
           p_Data = replace(p_Data, "dotr.Maia.Model.PlaceHolder","<<AppPackage>>.Model.<<ModelClass>>")
           p_Data = replace(p_Data, "PlaceHolder1","<<ModelClass>>1")
           
           p_Data = replace(p_Data,"PlaceHolder",ObjectConfig1:ClassName)
           p_Data = replace(p_Data,"dotr/Maia/Template/BaseUpdate",lv_UpdateBase)
       
           p_Data = replace(p_Data,"dotr.Maia.Template.BaseUpdate",replace(lv_UpdateBase,"/","."))
           p_Data = replace(p_Data," BaseUpdate ",substitute(" &1 ",ObjectConfig1:CurrentClassName))
                  
           p_Data = replace(p_Data,'THIS-OBJECT:groupControl1:Text = "groupControl1".',
                                   '~{&! UITextAddControl !}~n' +
                                   'THIS-OBJECT:groupControl1:Text = "groupControl1".~n' +
                                   '~{&! UITextBeginInit !}~n' +
                                   '~{&! UISetProperties !}~n')
           
           p_Data = replace(p_Data,"THIS-OBJECT:ResumeLayout(FALSE).","~{&! UITextEndInit !}~nTHIS-OBJECT:ResumeLayout(FALSE).")
           .
           
    copy-lob from file search("dotr/Maia/Template/BaseUpdate.resx") to lv_resx.
    copy-lob from lv_Resx to file substitute("&1.resx",lv_UpdateBase).
    
    return p_Data.
  end method.
  
  /** get statement for define variable
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void UITextDefine(p_Directive as Directive):
    assign p_Directive:Data = trim(textDefine,"~n").
  end method.
  
  /** get statement for new object 
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void UITextNew(p_Directive as Directive):
    assign p_Directive:Data = trim(textNew,"~n").
  end method.

  /** get statement for begininit
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void UITextBeginInit(p_Directive as Directive):
    assign p_Directive:Data = trim(BeginInit,"~n").
  end method.

  /** get statement for adding controls
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void UITextAddControl(p_Directive as Directive):
    assign p_Directive:Data = trim(AddControl,"~n").
  end method.

  /** get statement for define variable
   * this is the "chunky" part. All properties are drawn according to the parameters defined
   * in this properties update screen. All positions are calculated relative to the previously
   * drawn property. Labels are also defined automatically for each property unless the property
   * does not require them (radio-sets, or checkboxes for example)
   *
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void UISetProperties(p_Directive as Directive):
    def var lv_Data as char no-undo.
    
    def var lv_i   as int no-undo.
    def var lv_Max as int no-undo.
    
    def var font1 as System.Drawing.Font no-undo.
    def var size1 as System.Drawing.SizeF no-undo.
    
    def var Property1       as dotr.Maia.Model.Property       no-undo.
    def var DrawingOptions1 as dotr.Maia.Model.DrawingOptions no-undo.
    
    font1 = new System.Drawing.Font("Tahoma",8.25).
    
    DrawingOptions1 = new dotr.Maia.Model.DrawingOptions().
    
    /** first find the maximum label size. This needs to take into account proportional fonts */
    for each TTProperty no-lock where TTProperty.DisplayInForm gt 0 by TTProperty.PropertyName lv_i = 0 to lv_i + 1:
      Property1 = cast(TTProperty.Property,Property).

      if Property1:UIType eq "checkbox" then next.

      lv_Max = max(System.Windows.Forms.TextRenderer:MeasureText(Property1:FormLabel,font1):Width,lv_max).
    end.
        
    /** initial starting point to draw components */
    assign DrawingOptions1:ComponentDrawingPointY = 9
           DrawingOptions1:ComponentTabIndex      = 1.
    
    /** now loop through all properties to be drawn on the form */
    for each TTProperty no-lock where TTProperty.DisplayInForm gt 0 by TTProperty.DisplayInForm by TTProperty.PropertyName lv_i = 0 to lv_i + 1:
      Property1 = cast(TTProperty.Property,Property).
      
      assign DrawingOptions1:LabelDrawingPointX     = 10 + (lv_Max - (System.Windows.Forms.TextRenderer:MeasureText(Property1:FormLabel,font1):Width))
             DrawingOptions1:ComponentDrawingPointX = lv_max
             DrawingOptions1:ComponentHeight        = 1
             DrawingOptions1:ComponentLabel         = Property1:FormLabel
             DrawingOptions1:ComponentName          = replace(Property1:PropertyName,"-","")
             DrawingOptions1:ComponentTabIndex      = DrawingOptions1:ComponentTabIndex + 1
             DrawingOptions1:ComponentWidth         = Property1:WidthScale * 100
             DrawingOptions1:LabelDrawingPointY     = DrawingOptions1:ComponentDrawingPointY + 1.
      
      assign lv_Data = lv_Data + UIBuilder:ComponentBuilder(Property1:UIType):Properties(DrawingOptions1).
    end.
    
    assign p_Directive:Data = trim(lv_Data,"~n").
  end method.

  /** get statement for endinit
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void UITextEndInit(p_Directive as Directive):
    assign p_Directive:Data = trim(EndInit,"~n").
  end method.
  
  /** assign UI component screen values to properties
   * takes all of the field values and assigns them to the appropriate property.
   * The standard used for this is that all screen components are given the 
   * same name as the property itself, with "ui" prepended
   * 
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void UIAssignToObject(p_Directive as Directive):
    def var lv_Data   as char no-undo.
    def var lv_Assign as char no-undo.
    
    def var lv_maxLen as int no-undo.
    
    def var Property1 as Property no-undo.
    
    /** work out max size of property. for code alignment */
    for each TTProperty no-lock where TTProperty.DisplayInForm gt 0:
      Property1 = cast(TTProperty.Property,Property).
      
      if Property1:BuildProperty then assign lv_MaxLen = MAX(lv_MaxLen,TTProperty.SIZE).
    end.
    
    /** build the assign statement */
    for each TTProperty no-lock where TTProperty.DisplayInForm gt 0 :
      Property1 = cast(TTProperty.Property,Property).
      
      if not Property1:BuildProperty then next.
      if Property1:ManualProperty then next.
      
      assign lv_Assign = substitute("<<ModelClass>>1:&1@ = &2ui&4:Text&3~n",
                                    Property1:PropertyName,
                                    if TTProperty.DataType eq "character" then "" else TTProperty.DataType + "(",
                                    if TTProperty.DataType eq "character" then "" else ")",
                                    replace(Property1:PropertyName,"-",""))
                                    
             lv_Data   = lv_Data + replace(lv_Assign,"@",fill(" ",lv_maxLen - TTProperty.Size)). 
    end.

    assign p_Directive:Data = trim(lv_Data,"~n").                                                                                                                                                  
  end method.
  
  /** assign properties to UI component screen values 
   * takes properties and assigns them to the appropriate field values 
   * The standard used for this is that all screen components are given the 
   * same name as the property itself, with "ui" prepended
   * 
   * @param p_Directive : holds the directive name and all parameters
   */

  method public void  UIAssignToUI(p_Directive as Directive):
    def var lv_Data   as char no-undo.
    def var lv_Assign as char no-undo.
    
    def var lv_maxLen as int no-undo.
    
    def var Property1 as Property no-undo.
    
    /** work out max size of property. for code alignment */
    for each TTProperty no-lock where TTProperty.DisplayInForm gt 0:
      Property1 = cast(TTProperty.Property,Property).
      
      if Property1:BuildProperty then assign lv_MaxLen = MAX(lv_MaxLen,TTProperty.SIZE).
    end.
    
    /** build the assign statement */
    for each TTProperty no-lock where TTProperty.DisplayInForm gt 0:
      Property1 = cast(TTProperty.Property,Property).
      
      if not Property1:BuildProperty then next.
      if Property1:ManualProperty then next.
      
      assign lv_Assign = substitute("ui&4:text@ = &2<<ModelClass>>1:&1&3~n",
                                    Property1:PropertyName,
                                    if TTProperty.DataType eq "character" then "" else "string(",
                                    if TTProperty.DataType eq "character" then "" else ")",
                                    replace(Property1:PropertyName,"-",""))
             lv_Data   = lv_Data + replace(lv_Assign,"@",fill(" ",lv_maxLen - TTProperty.Size)). 
    end.

    assign p_Directive:Data = trim(lv_Data,"~n").                                                                                                                                                  
  end method.
  

end class.