/*
Copyright (c) 2011, Julian Lyndon-Smith (julian+maia@dotr.com)
http://www.dotr.com
All rights reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

using Progress.Lang.*.
using dotr.Maia.Model.*.

/** Properties and methods for ObjectConfig 
 * @author Julian Lyndon-Smith (julian+maia@dotr.com) 
 */

class dotr.Maia.Model.ObjectConfig: 
  def public property ClassName         as char no-undo get . set.
  def public property TableName         as char no-undo get . set.
  def public property ModelName         as char no-undo get . set.
  def public property ModelPackage      as char no-undo get . set.
  def public property ConfigFile        as char no-undo get . set.
  def public property KeyFieldName      as char no-undo get . set.
  def public property KeyProperty       as char no-undo get . set.
  def public property KeyFieldDatatype  as char no-undo get . set.
  def public property MD5Hash           as char no-undo get . set.
  def public property Outputfile        as char no-undo get . set.
  def public property CurrentClassName  as char no-undo get . set . 
  def public property fkDelete          as char no-undo get . set .
  def public property LogicalDb         as char no-undo get . set .
  
  def public property Data as longchar extent no-undo get . set.
    
  def public property BuildObject as logical no-undo get . set.
  def public property isAbstract  as logical no-undo get . set.
  
  def public property NumProperties as  int  no-undo get . set.
  def public property NumTemplates  as  int  no-undo get . set.
  def public property NumIndex      as  int  no-undo get . set.
  def public property NumCustomCode as  int  no-undo get . set.
  def public property CRC           as  int  no-undo get . set.
  
  def public property Property   as property   extent 256 no-undo get . set.
  def public property Template   as Template   extent 64  no-undo get . set.
  def public property CustomCode as CustomCode extent 64  no-undo get . set.
  def public property index      as index      extent 64  no-undo get . set.
  
  def public property ObjectBuilder as dotr.Maia.Builder.ObjectBuilder no-undo get . public set.
  
  
  /** Saves this instance 
   */

  method public void Save():
    (NEW dotr.Maia.Crud.ObjectConfig()):Save(this-object).
  end method.

  /** returns a populated ObjectConfig based on the config file
   * @param p_ConfigFile : Name of the config to load
   * @return ObjectConfig : ObjectConfig instance
   */

  method public static ObjectConfig Get(p_ConfigFile as char):
    return (new dotr.Maia.Crud.ObjectConfig()):Get(p_ConfigFile).
  end method.

  /** returns a Property 
   * @param p_Property : Name of the property to get
   * @return Property : property instance
   */
  method public property GetProperty(p_Property as char):
    return (new dotr.Maia.Crud.ObjectConfig()):GetProperty(this-object,p_Property).
  end method.

  /** returns a Property 
   * @param p_FieldName : Name of the property to get
   * @return Property : property instance
   */

  method public property GetFieldName(p_FieldName as char):
    return (new dotr.Maia.Crud.ObjectConfig()):GetFieldName(this-object,p_FieldName).
  end method.

  /** adds a Property 
   * @param p_Property : property instance to add
   */

  method public void AddProperty(p_Property as Property):
    (NEW dotr.Maia.Crud.ObjectConfig()):AddProperty(this-object,p_Property).
  end method.

  /** returns a CustomCode 
   * @param p_CustomCode : Name of the CustomCode to get
   * @return CustomCode : CustomCode instance
   */

  method public CustomCode GetCustomCode(p_CustomCode as char):
    return (new dotr.Maia.Crud.ObjectConfig()):GetCustomCode(this-object,p_CustomCode).
  end method.

  /** adds a CustomCode 
   * @param p_CustomCode : CustomCode instance to add
   */

  method public void AddCustomCode(p_CustomCode as CustomCode):
    (NEW dotr.Maia.Crud.ObjectConfig()):AddCustomCode(this-object,p_CustomCode).
  end method.

  /** returns a Index 
   * @param p_Index : Name of the Index to get
   * @return Index : Index instance
   */

  method public Index GetIndex(p_Index as char):
    return (new dotr.Maia.Crud.ObjectConfig()):GetIndex(this-object,p_Index).
  end method.

  /** adds a Index 
   * @param p_Index : Index instance to add
   */

  method public void AddIndex(p_Index as Index):
    (NEW dotr.Maia.Crud.ObjectConfig()):AddIndex(this-object,p_Index).
  end method.

  /** returns a Template 
   * @param p_Template : Name of the Template to get
   * @return Template : Template instance
   */

  method public Template GetTemplate(p_Template as char):
    return (new dotr.Maia.Crud.ObjectConfig()):GetTemplate(this-object,p_Template).
  end method.

  /** adds a Template 
   * @param p_Template : Template instance to add
   */

  method public void AddTemplate(p_Template as Template):
    (NEW dotr.Maia.Crud.ObjectConfig()):AddTemplate(this-object,p_Template).
  end method.

  /** adds a Template 
   * @param p_Template : Template instance to add
   */

  method public void AddTemplate(p_Template as char):
    (NEW dotr.Maia.Crud.ObjectConfig()):AddTemplate(this-object,p_Template).
  end method.
  
end class.