
using Progress.Lang.*.
using dotr.Maia.Stomp.*.
using dotr.Maia.Stomp.Interface.*.

routine-level on error undo, throw.

class dotr.Maia.Stomp.StompConsumer :
  def temp-table TTDestination no-undo
    field Destination as char
    field ID as char
    
    index Dest is primary unique
      Destination.

  def var StompConnection as dotr.Maia.Stomp.StompConnection no-undo.
  
  constructor StompConsumer(p_StompServer as StompServer,p_Handler as handle):
    StompConnection = new dotr.Maia.Stomp.StompConnection(p_StompServer,p_Handler).
  end constructor.

  constructor StompConsumer(p_StompServer as StompServer,p_Handler as dotr.Maia.Stomp.Interface.IStompHandler):
    StompConnection = new dotr.Maia.Stomp.StompConnection(p_StompServer,p_Handler).
  end constructor.

  destructor StompConsumer():
    for each TTDestination no-lock:
      this-object:unsubscribeID(TTDestination.Destination,TTDestination.ID).
      delete TTDestination.
    end.
  end destructor.
 
  method public void SubscribeToQueue(p_Destination as char):
    this-object:Subscribe("queue",p_Destination).
  end method.

  method public void SubscribeToTopic(p_Destination as char):
    this-object:Subscribe("topic",p_Destination).
  end method.

  method public void UnSubscribeFromQueue(p_Destination as char):
    this-object:UnSubscribe("queue",p_Destination).
  end method.

  method public void UnSubscribeFromTopic(p_Destination as char):
    this-object:UnSubscribe("topic",p_Destination).
  end method.

  method private void Subscribe(p_type as char,p_Destination as char):
    def var lv_i as int no-undo.
    
    do lv_I = 1 to num-entries(p_Destination):
      find TTDestination where TTDestination.Destination eq entry(lv_i,p_Destination) no-lock no-error.
      if avail TTDestination then return.
      
      create TTDestination.
      assign TTDestination.Destination = substitute("/&1/&2",p_type,entry(lv_i,p_Destination))
             TTDestination.ID          = guid(generate-uuid).
             
      StompConnection:sendFrame(substitute("SUBSCRIBE~nid:&1~nactivemq.prefetchSize: 1~ndestination: &2~nack: &3~n~n", TTDestination.ID,TTDestination.Destination, "auto")).
    end.                 
  end method.
  
  method private void UnSubscribe(p_type as char,p_Destination as char):
    find TTDestination where TTDestination.Destination eq substitute("/&1/&2",p_type,p_Destination) no-lock no-error.
    if not avail TTDestination then return.  
    this-object:unsubscribeID(TTDestination.ID,TTDestination.Destination).
  end method.

  method private void UnSubscribeID(p_Destination as char,p_ID as char):
    StompConnection:sendFrame(substitute("UNSUBSCRIBE~nid:&1destination: &2~n~n", p_ID,p_Destination)).
  end method.

end class.