
/*
Copyright (C) 2011 by Julian Lyndon-Smith (julian+mit@dotr.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS for A PARTICULAR PURPOSE and NONINFRINGEMENT. IN no EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE for ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR otherwise, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

  
using Progress.Lang.*.
using Progress.Data.*.
using sports.helper.*.
using sports.model.*.

routine-level on error undo, throw.

class sports.Helper.Base.Customer:
  
  /** gets a Customer using FindUsingComments
   *
   * @param  p_Comments - character
   * @return sports.Model.Customer
   */
  method public static sports.Model.Customer FindUsingComments( p_Comments as character):
    return FindWhere("Customer.Comments eq '" + p_Comments + "'").
  end method.
  
  /** gets a Customer using FindUsingCountryPostalCode
   *
   * @param  p_Country - character, p_PostalCode - character
   * @return sports.Model.Customer
   */
  method public static sports.Model.Customer FindUsingCountryPostalCode( p_Country as character, p_PostalCode as character):
    return FindWhere("Customer.Country eq '" + p_Country + "' and Customer.Postal-Code eq '" + p_PostalCode + "'").
  end method.
  
  /** gets a Customer using FindUsingCustNum
   *
   * @param  p_CustNum - integer
   * @return sports.Model.Customer
   */
  method public static sports.Model.Customer FindUsingCustNum( p_CustNum as integer):
    return FindWhere("Customer.Cust-Num eq " + string(p_CustNum) + "").
  end method.
  
  /** gets a Customer using FindUsingName
   *
   * @param  p_Name - character
   * @return sports.Model.Customer
   */
  method public static sports.Model.Customer FindUsingName( p_Name as character):
    return FindWhere("Customer.Name eq '" + p_Name + "'").
  end method.
  
  /** gets a Customer using FindUsingSalesRep
   *
   * @param  p_SalesRep - character
   * @return sports.Model.Customer
   */
  method public static sports.Model.Customer FindUsingSalesRep( p_SalesRep as character):
    return FindWhere("Customer.Sales-Rep eq '" + p_SalesRep + "'").
  end method.
  
    
  method public static sports.Model.Customer FindWhere(p_Query as char):
    return (new sports.Model.Customer()):FindWhere(p_Query).
  end method.
  
  /** gets the Customer matching the unique ID 
   *  @param p_CustomerID : the id to find
   */

  method public static sports.Model.Customer find(p_CustomerID as Integer):
    return (new sports.Model.Customer()):FindWhere(substitute("Customer.Cust-Num eq '&1'",p_CustomerID)).
  end.
  
  /** open a query for all records in the <<Table>> table
   */
  method public static sports.Model.Customer FindAll():
    return (new sports.Model.Customer()):FindWhere("").
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.Customer  
   */
  
  method public static sports.Model.Customer FindWhere (p_BindingSource as BindingSource,p_Query as char):
    return (new sports.Model.Customer()):FindWhere(p_BindingSource,p_Query).
  end method.

  /** clears all Customer with a foreign key
   *
   * @param p_fkGUID = Foreign Key
   */
  
  method public static void StateDeleted(p_fkGUID as char):
    def var Customer1 as sports.Model.Customer no-undo.
  
    Customer1 = getState(p_fkGUID).
    
    do while Customer1:Available:
      Customer1:AssignState('').
      Customer1:next().
    end.
  
  end method.
  
  
  /** get all Customer associated with a foreign key
   *
   * @param p_fkGUID = Foreign Key
   */
  
    method public static sports.Model.Customer getState(p_fkGUID as char):
      return (new sports.Model.Customer()):FindWhere(substitute("State eq '&1'",p_fkGUID)).
    end method.
  

end class.
@BuildInfo(Source="./sports/_Maia-Config/Customer/_Config.xml").
@BuildInfo(Date="30/08/2011 08:37:15.098+01:00").