
/*
Copyright (C) 2011 by Julian Lyndon-Smith (julian+mit@dotr.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS for A PARTICULAR PURPOSE and NONINFRINGEMENT. IN no EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE for ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR otherwise, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/


using Progress.Lang.*.
using sports.helper.*.
using sports.model.*.
using Progress.Data.*.

routine-level on error undo, throw.

class sports.Model.Base.Customer:
  def buffer Customer for sports.Customer.

  def public property rowid as rowid  get. set. /* must be undo, as any error on creation must be reset to ? */
  def public property Query as handle no-undo get . protected set .     
  
  def protected property buffer     as handle no-undo get . protected set .
  def protected property PrimaryKey as handle no-undo get . protected set .
  
  def public property PrivateData as char no-undo get. set.
  
  def public property Available as logical no-undo
    get():
      return rowid ne ?.
    end get . protected set.
    
  def public property NumRecords as int no-undo
    get():
      return query:num-results.
    end get . protected set.

  def public property Address    as character no-undo  get . set . 
  def public property Address2   as character no-undo  get . set . 
  def public property City       as character no-undo  get . set . 
  def public property Comments   as character no-undo  get . set . 
  def public property Contact    as character no-undo  get . set . 
  def public property Country    as character no-undo  get . set . 
  def public property Name       as character no-undo  get . set . 
  def public property Phone      as character no-undo  get . set . 
  def public property PostalCode as character no-undo  get . set . 
  def public property SalesRep   as character no-undo  get . set . 
  def public property State      as character no-undo  get . set . 
  def public property Terms      as character no-undo  get . set . 
  
  def public property Balance     as decimal no-undo  get . set . 
  def public property CreditLimit as decimal no-undo  get . set . 
  
  def public property CustNum  as integer no-undo  get . set . 
  def public property Discount as integer no-undo  get . set . 
  
  def public property StateObject as sports.Model.State no-undo  get . set . 
  
  
  constructor Customer():
    StartUp().
  end constructor.


  /** Creates the buffer handle and buffer fields 
   */

  method protected void StartUp():
    create query this-object:Query.
    
    assign this-object:buffer = buffer Customer:handle
           this-object:PrimaryKey = this-object:buffer:buffer-field("Cust-Num"):handle.
            
    this-object:Query:add-buffer (this-object:buffer).
  end method.

  /** gets the Customer matching the unique ID 
   *  @param p_CustomerID : the id to find
   */

  method public sports.Model.Customer find(p_CustomerID as Integer):
    return this-object:FindWhere(substitute("Customer.Cust-Num eq '&1'",p_CustomerID)).
  end.

  /** stub method to allow for override
   */

  method public void AfterGet():
  end method.

 /** stub method to allow for override
   */

  method public void BeforeGet():
  end method.

 /** stub method to allow for override
  */

  method public void BeforeCreate():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterCreate():
  end method.

 /** stub method to allow for override
  */

  method public void BeforeUpdate():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterUpdate():
  end method.
  
 /** stub method to allow for override
  */

  method public void BeforeRemove():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterRemove():
  end method.

 /** stub method to allow for override
  */
  method public void NotFound():
  end method.

  /** assigns the buffer fields to the object
   *  @return sports.Model.Customer  
   */

  method protected sports.Model.Customer get():
    BeforeGet().
    assign this-object:rowid = ?.
        
    if not this-object:buffer:available then
    do: 
      NotFound().
      return cast(this-object,sports.Model.Customer). 
    end.

    assign this-object:Address     = buffer::Address
           this-object:Address2    = buffer::Address2
           this-object:City        = buffer::City
           this-object:Comments    = buffer::Comments
           this-object:Contact     = buffer::Contact
           this-object:Country     = buffer::Country
           this-object:Name        = buffer::Name
           this-object:Phone       = buffer::Phone
           this-object:PostalCode  = buffer::Postal-Code
           this-object:SalesRep    = buffer::Sales-Rep
           this-object:State       = buffer::State
           this-object:Terms       = buffer::Terms
           this-object:Balance     = buffer::Balance
           this-object:CreditLimit = buffer::Credit-Limit
           this-object:CustNum     = buffer::Cust-Num
           this-object:Discount    = buffer::Discount
           this-object:rowid       = buffer:rowid.
           
    AfterGet().
               
    return cast(this-object,sports.Model.Customer). 
  end.
  
  /** open a query for all records in the <<Table>> table
   */
  method public sports.Model.Customer all():
    return this-object:FindWhere("").
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.Customer  
   */
  
  method public sports.Model.Customer FindWhere (p_BindingSource as BindingSource,p_Where as char):
    this-object:FindWhere(p_Where).
    p_BindingSource:handle = this-object:query.
    return cast(this-object,sports.Model.Customer). 
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.Customer  
   */
  
  method public sports.Model.Customer FindWhere (p_Where as char):
    this-object:Query:query-prepare(substitute("for each Customer &1 no-lock",
                                                if trim(p_Where) begins "where " 
                                                   then p_Where
                                                   else if p_Where ne "" then substitute("where &1",p_Where)
                                                   else "")).
    this-object:Query:query-open().
    return this-object:First().                                                        
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.Customer  
   */
  
  method public sports.Model.Customer preselect (p_Where as char):
    this-object:Query:query-prepare(substitute("preselect each Customer &1 no-lock",
                                                if trim(p_Where) begins "where " 
                                                   then p_Where
                                                   else if p_Where ne "" then substitute("where &1",p_Where)
                                                   else "")).
    this-object:Query:query-open().
    return this-object:First().                                                        
  end method.
  
  /** gets the first Customer of the query
   *  @return sports.Model.Customer  
   */

  method public sports.Model.Customer first():
    this-object:Query:get-first().
    return this-object:Get().
  end method.
    
  /** gets the last Customer of the query
   *  @return sports.Model.Customer  
   */

  method public sports.Model.Customer last():
    this-object:Query:get-last().
    return this-object:Get().
  end method.

  /** gets the next Customer of the query
   *  @return sports.Model.Customer  
   */

  method public sports.Model.Customer next():
    this-object:Query:get-next().
    return this-object:Get().
  end method.

  /** gets the prev Customer of the query
   *  @return sports.Model.Customer  
   */

  method public sports.Model.Customer prev():
    this-object:Query:get-prev().
    return this-object:Get().
  end method.

  /** saves the object attributes to the database
   *  @return sports.Model.Customer  
   */
  
  method public sports.Model.Customer save():
    WriteRecord(if this-object:rowid eq ? then "CREATE" else "UPDATE").
    return cast(this-object,sports.Model.Customer). 
  end method.

  /** removes the object from the database
   */
 
  method public void Remove():
    if not this-object:available then return.
    WriteRecord("Remove").
    return. 
  end method. 

  /** writes the approriate changes to the database
   *  @param p_Mode : create (creates a new <<table>>
   *                : update (saves the changes to the database)
   *                : remove (removes the record from the database)
   */
    
  method protected final void WriteRecord(p_Mode as char):
    def var lv_new as logical no-undo.
    
    def buffer Customer for sports.Customer.
    
    assign lv_new = this-object:rowid eq ?.
    
    /** try to lock the record, give up after 5 seconds */
    repeat while true stop-after 5 transaction on error undo, throw
                                               on stop undo, leave:

      if p_mode eq "CREATE" then 
      do:
        BeforeCreate(). /** override this method if you want to do stuff before the create statement */ 

        create Customer.
        if this-object:CustNum eq 0 or this-object:CustNum eq ? then assign this-object:CustNum  = next-value(Next-Cust-Num,sports).
        
        assign this-object:rowid = rowid(Customer).
               
        AfterCreate().  /** override this method if you want to do stuff after the create statement */
      end.
      
      else 
      do:
        find Customer where Customer.Cust-Num eq this-object:CustNum exclusive no-wait no-error.
   
        if locked(Customer) then next.
        if not avail Customer then undo, throw new AppError(substitute("Could not find Customer [&1]",this-object:CustNum),0).
      end.
      
      case p_Mode:
        when "Update" or when "Create" then
        do:
          BeforeUpdate().  /** override this method if you want to do stuff before the update statement */
          assign Customer.Address      = this-object:Address
                 Customer.Address2     = this-object:Address2
                 Customer.City         = this-object:City
                 Customer.Comments     = this-object:Comments
                 Customer.Contact      = this-object:Contact
                 Customer.Country      = this-object:Country
                 Customer.Name         = this-object:Name
                 Customer.Phone        = this-object:Phone
                 Customer.Postal-Code  = this-object:PostalCode
                 Customer.Sales-Rep    = this-object:SalesRep
                 Customer.State        = this-object:State
                 Customer.Terms        = this-object:Terms
                 Customer.Balance      = this-object:Balance
                 Customer.Credit-Limit = this-object:CreditLimit
                 Customer.Cust-Num     = this-object:CustNum
                 Customer.Discount     = this-object:Discount.
          AfterUpdate().  /** override this method if you want to do stuff after the update statement */
        end.
        
        when "Remove" then
        do:
          BeforeRemove().  /** override this method if you want to do stuff before the remove statement */
          delete Customer.
          assign this-object:rowid = ?.


          AfterRemove().  /** override this method if you want to do stuff after the remove statement */
        end.
        
      end case.
      
      dotr.Maia.Stomp.Helper.SendMessage:ToTopic("sports.info.db.update.Customer",substitute("action:&1,guid:&2",p_mode,this-object:CustNum)).
      
      release Customer.
      
      leave.
      
      catch e as Progress.Lang.Error :
        case e:GetMessageNum(1):
          when 132 then undo, throw new AppError(e:getMessage(1),132).
          otherwise     undo, throw new AppError(e:GetMessage(1),0).
        end case.
      end catch.
    end.
    
    return.
    
  end method.    

  /** Assigns Customer with a foreign key
   *
   * @param p_fkGUID = Foreign Key
   */
  
  method public void AssignState(p_fkGUID as char):
    assign this-object:State = p_fkGUID.
    this-object:save().
  end method.
  
end class.
@BuildInfo(Source="./sports/_Maia-Config/Customer/_Config.xml").
@BuildInfo(Date="30/08/2011 08:37:15.077+01:00").