
/*
Copyright (C) 2011 by Julian Lyndon-Smith (julian+mit@dotr.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS for A PARTICULAR PURPOSE and NONINFRINGEMENT. IN no EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE for ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR otherwise, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/


using Progress.Lang.*.
using sports.helper.*.
using sports.model.*.
using Progress.Data.*.

routine-level on error undo, throw.

class sports.Model.Base.State:
  def buffer State for sports.State.

  def public property rowid as rowid  get. set. /* must be undo, as any error on creation must be reset to ? */
  def public property Query as handle no-undo get . protected set .     
  
  def protected property buffer     as handle no-undo get . protected set .
  def protected property PrimaryKey as handle no-undo get . protected set .
  
  def public property PrivateData as char no-undo get. set.
  
  def public property Available as logical no-undo
    get():
      return rowid ne ?.
    end get . protected set.
    
  def public property NumRecords as int no-undo
    get():
      return query:num-results.
    end get . protected set.

  def public property Region    as character no-undo  get . set . 
  def public property State     as character no-undo  get . set . 
  def public property StateName as character no-undo  get . set . 
  
  
  constructor State():
    StartUp().
  end constructor.


  /** Creates the buffer handle and buffer fields 
   */

  method protected void StartUp():
    create query this-object:Query.
    
    assign this-object:buffer = buffer State:handle
           this-object:PrimaryKey = this-object:buffer:buffer-field("State"):handle.
            
    this-object:Query:add-buffer (this-object:buffer).
  end method.

  /** gets the State matching the unique ID 
   *  @param p_StateID : the id to find
   */

  method public sports.Model.State find(p_StateID as Character):
    return this-object:FindWhere(substitute("State.State eq '&1'",p_StateID)).
  end.

  /** stub method to allow for override
   */

  method public void AfterGet():
  end method.

 /** stub method to allow for override
   */

  method public void BeforeGet():
  end method.

 /** stub method to allow for override
  */

  method public void BeforeCreate():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterCreate():
  end method.

 /** stub method to allow for override
  */

  method public void BeforeUpdate():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterUpdate():
  end method.
  
 /** stub method to allow for override
  */

  method public void BeforeRemove():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterRemove():
  end method.

 /** stub method to allow for override
  */
  method public void NotFound():
  end method.

  /** assigns the buffer fields to the object
   *  @return sports.Model.State  
   */

  method protected sports.Model.State get():
    BeforeGet().
    assign this-object:rowid = ?.
        
    if not this-object:buffer:available then
    do: 
      NotFound().
      return cast(this-object,sports.Model.State). 
    end.

    assign this-object:Region    = buffer::Region
           this-object:State     = buffer::State
           this-object:StateName = buffer::State-Name
           this-object:rowid     = buffer:rowid.
           
    AfterGet().
               
    return cast(this-object,sports.Model.State). 
  end.
  
  /** open a query for all records in the <<Table>> table
   */
  method public sports.Model.State all():
    return this-object:FindWhere("").
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.State  
   */
  
  method public sports.Model.State FindWhere (p_BindingSource as BindingSource,p_Where as char):
    this-object:FindWhere(p_Where).
    p_BindingSource:handle = this-object:query.
    return cast(this-object,sports.Model.State). 
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.State  
   */
  
  method public sports.Model.State FindWhere (p_Where as char):
    this-object:Query:query-prepare(substitute("for each State &1 no-lock",
                                                if trim(p_Where) begins "where " 
                                                   then p_Where
                                                   else if p_Where ne "" then substitute("where &1",p_Where)
                                                   else "")).
    this-object:Query:query-open().
    return this-object:First().                                                        
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.State  
   */
  
  method public sports.Model.State preselect (p_Where as char):
    this-object:Query:query-prepare(substitute("preselect each State &1 no-lock",
                                                if trim(p_Where) begins "where " 
                                                   then p_Where
                                                   else if p_Where ne "" then substitute("where &1",p_Where)
                                                   else "")).
    this-object:Query:query-open().
    return this-object:First().                                                        
  end method.
  
  /** gets the first State of the query
   *  @return sports.Model.State  
   */

  method public sports.Model.State first():
    this-object:Query:get-first().
    return this-object:Get().
  end method.
    
  /** gets the last State of the query
   *  @return sports.Model.State  
   */

  method public sports.Model.State last():
    this-object:Query:get-last().
    return this-object:Get().
  end method.

  /** gets the next State of the query
   *  @return sports.Model.State  
   */

  method public sports.Model.State next():
    this-object:Query:get-next().
    return this-object:Get().
  end method.

  /** gets the prev State of the query
   *  @return sports.Model.State  
   */

  method public sports.Model.State prev():
    this-object:Query:get-prev().
    return this-object:Get().
  end method.

  /** saves the object attributes to the database
   *  @return sports.Model.State  
   */
  
  method public sports.Model.State save():
    WriteRecord(if this-object:rowid eq ? then "CREATE" else "UPDATE").
    return cast(this-object,sports.Model.State). 
  end method.

  /** removes the object from the database
   */
 
  method public void Remove():
    if not this-object:available then return.
    WriteRecord("Remove").
    return. 
  end method. 

  /** writes the approriate changes to the database
   *  @param p_Mode : create (creates a new <<table>>
   *                : update (saves the changes to the database)
   *                : remove (removes the record from the database)
   */
    
  method protected final void WriteRecord(p_Mode as char):
    def var lv_new as logical no-undo.
    
    def buffer State for sports.State.
    
    assign lv_new = this-object:rowid eq ?.
    
    /** try to lock the record, give up after 5 seconds */
    repeat while true stop-after 5 transaction on error undo, throw
                                               on stop undo, leave:

      if p_mode eq "CREATE" then 
      do:
        BeforeCreate(). /** override this method if you want to do stuff before the create statement */ 

        create State.
        
        assign this-object:rowid = rowid(State).
               
        AfterCreate().  /** override this method if you want to do stuff after the create statement */
      end.
      
      else 
      do:
        find State where State.State eq this-object:State exclusive no-wait no-error.
   
        if locked(State) then next.
        if not avail State then undo, throw new AppError(substitute("Could not find State [&1]",this-object:State),0).
      end.
      
      case p_Mode:
        when "Update" or when "Create" then
        do:
          BeforeUpdate().  /** override this method if you want to do stuff before the update statement */
          assign State.Region     = this-object:Region
                 State.State      = this-object:State
                 State.State-Name = this-object:StateName.
          AfterUpdate().  /** override this method if you want to do stuff after the update statement */
        end.
        
        when "Remove" then
        do:
          BeforeRemove().  /** override this method if you want to do stuff before the remove statement */
          delete State.
          assign this-object:rowid = ?.

          sports.Helper.Customer:StateDeleted(this-object:State).

          AfterRemove().  /** override this method if you want to do stuff after the remove statement */
        end.
        
      end case.
      
      dotr.Maia.Stomp.Helper.SendMessage:ToTopic("sports.info.db.update.State",substitute("action:&1,guid:&2",p_mode,this-object:State)).
      
      release State.
      
      leave.
      
      catch e as Progress.Lang.Error :
        case e:GetMessageNum(1):
          when 132 then undo, throw new AppError(e:getMessage(1),132).
          otherwise     undo, throw new AppError(e:GetMessage(1),0).
        end case.
      end catch.
    end.
    
    return.
    
  end method.    

end class.
@BuildInfo(Source="./sports/_Maia-Config/State/_Config.xml").
@BuildInfo(Date="30/08/2011 08:37:31.925+01:00").